package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.AreaDto;
import com.qmth.sop.business.bean.params.ServiceRegionParam;
import com.qmth.sop.business.bean.result.CrmServiceResult;
import com.qmth.sop.business.bean.result.ServiceRegionResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.entity.TBServiceRegion;
import com.qmth.sop.business.entity.TBServiceRegionDetail;
import com.qmth.sop.business.mapper.TBServiceRegionMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.RoleTypeEnum;
import com.qmth.sop.common.enums.ServiceStatusEnum;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务区域规划表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBServiceRegionServiceImpl extends ServiceImpl<TBServiceRegionMapper, TBServiceRegion> implements TBServiceRegionService {
    @Resource
    private TBServiceService tbServiceService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private TBServiceRegionDetailService tbServiceRegionDetailService;
    @Resource
    private SysUserRoleService sysUserRoleService;
    @Resource
    private TBCrmService tbCrmService;

    @Override
    public IPage<ServiceRegionResult> findServiceRegionPage(Long serviceUnitId, Long leadId, String city, Integer pageNumber, Integer pageSize) {
        if (city != null && city.length() > 0) {
            city = SystemConstant.translateSpecificSign(city);
        }
        IPage<ServiceRegionResult> page = this.baseMapper.findServiceRegionPage(new Page<>(pageNumber, pageSize), serviceUnitId, leadId, city, ServiceStatusEnum.CANCEL);
        List<ServiceRegionResult> records = page.getRecords();
        if (records.size() > 0) {
            // 涉及的服务单元id集合
            List<Long> serviceUnitIdList = records.stream().map(ServiceRegionResult::getServiceUnitId).distinct().collect(Collectors.toList());

            // 派单
            List<CrmServiceResult> crmServiceResultList = tbCrmService.findCrmOrderByServiceIdList(serviceUnitIdList);
            // 规划位置
            List<Long> serviceRegionIdList = records.stream().map(ServiceRegionResult::getServiceRegionId).distinct().collect(Collectors.toList());
            List<TBServiceRegionDetail> tbServiceRegionDetailList = tbServiceRegionDetailService.list(new QueryWrapper<TBServiceRegionDetail>().lambda().in(TBServiceRegionDetail::getServiceRegionId, serviceRegionIdList));

            for (ServiceRegionResult record : records) {
                Long unitId = record.getServiceUnitId();
                Long regionId = record.getServiceRegionId();

                List<TBServiceRegionDetail> detailList = tbServiceRegionDetailList
                        .stream()
                        .filter(e -> regionId.equals(e.getServiceRegionId()))
                        .collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(detailList)) {
                    int orderCount = 0;
                    StringBuilder regionInfo = new StringBuilder();
                    for (TBServiceRegionDetail detail : detailList) {
                        String detailProvince = detail.getProvince();
                        String detailCity = detail.getCity();
                        String regionInfoCell = detailProvince + detailCity;

                        List<CrmServiceResult> crmList = crmServiceResultList.stream()
                                .filter(e -> unitId.equals(e.getServiceUnitId())
                                        && detailProvince.equals(e.getProvince())
                                        && detailCity.equals(e.getCity()))
                                .collect(Collectors.toList());
                        int count = crmList.size();
                        regionInfoCell = regionInfoCell + "(" + count + ")";
                        orderCount = orderCount + count;
                        regionInfo.append(regionInfoCell).append("、");
                    }
                    if (regionInfo.length() > 0) {
                        regionInfo = new StringBuilder(regionInfo.substring(0, regionInfo.length() - 1));
                    }
                    record.setOrderCount(orderCount);
                    record.setRegionInfo(regionInfo.toString());
                }
            }
        }
        return page;
    }

    @Transactional
    @Override
    public Long editServiceRegion(ServiceRegionParam serviceRegionParam) {
        Long serviceRegionId = serviceRegionParam.getServiceRegionId();
        Long serviceUnitId = serviceRegionParam.getServiceUnitId();
        Long leadId = serviceRegionParam.getLeadId();
        List<AreaDto> areaDtoList = serviceRegionParam.getAreaDtoList();

        TBService tbServiceUnit = tbServiceService.getById(serviceUnitId);
        if (Objects.isNull(tbServiceUnit)) {
            throw ExceptionResultEnum.ERROR.exception("服务单元不存在");
        }

        SysUser leader = sysUserService.getById(leadId);
        if (Objects.isNull(leader)) {
            throw ExceptionResultEnum.ERROR.exception("大区经理不存在");
        }
        if (!sysUserRoleService.userContainsRoles(leadId, RoleTypeEnum.REGION_MANAGER)) {
            throw ExceptionResultEnum.ERROR.exception("所选用户没有大区经理角色");
        }


        TBServiceRegion tbServiceRegion = new TBServiceRegion();
        tbServiceRegion.setServiceId(serviceUnitId);
        tbServiceRegion.setLeadId(leadId);

        // 更新区域规划信息
        if (Objects.isNull(serviceRegionId)) {
            // 新增
            tbServiceRegion.setEnable(true);
            this.save(tbServiceRegion);
            serviceRegionId = tbServiceRegion.getId();
        } else {
            // 编辑
            this.updateById(tbServiceRegion);
        }

        // 更新详细信息
        tbServiceRegionDetailService.bindServiceRegionAreas(serviceRegionId, areaDtoList);
        return serviceRegionId;
    }

    @Transactional
    @Override
    public void removeServiceRegion(Long serviceRegionId) {
        TBServiceRegion tbServiceRegion = this.getById(serviceRegionId);
        if (Objects.isNull(tbServiceRegion)) {
            throw ExceptionResultEnum.ERROR.exception("服务区域规划不存在");
        }
        tbServiceRegion.setEnable(false);
        this.updateById(tbServiceRegion);
    }
}
